# frozen_string_literal: true

require File.expand_path('../spec_helper', __dir__)

module Danger
  describe Danger::DangerWCC do
    before do
      @dangerfile = testing_dangerfile
      @my_plugin = @dangerfile.wcc
      @git = @dangerfile.git
      @github = @dangerfile.github

      allow(@github).to receive(:html_link) do |text|
        "<a href=\"github_html_link\">#{text}</a>"
      end

      allow(File).to receive(:exist?).and_call_original
      allow(File).to receive(:read).and_call_original
      allow(File).to receive(:exist?).with('yarn.lock')
        .and_return(true)
      allow(File).to receive(:readlines).with('yarn.lock')
        .and_return(load_fixture('dependencies/yarn.lock').split("\n"))
      allow(File).to receive(:read).with('package.json')
        .and_return(load_fixture('dependencies/package.json'))
      allow(Dir).to receive(:glob).and_return([])
    end

    describe 'yarn dependencies' do
      let(:subject) { Danger::DangerWCC::Dependencies.new(@my_plugin) }

      it 'Errors when minor version changes' do
        allow(subject).to receive(:run_and_diff)
          .with(/yarn list/)
          .and_return(load_fixture('dependencies/yarn_minor_version.diff'))
        allow(@git).to receive(:diff)
          .and_return([
                        load_diff(
                          'spec/fixtures/dependencies/package.json',
                          'dependencies/package.json'
                        )
                      ])

        # act
        subject.perform

        expect(@dangerfile.violation_report[:warnings])
          .to eq([Violation.new(
            'Dangerous change! react-instantsearch was updated '\
            'from 5.3.2 to 5.7.0 without a corresponding '\
            'change to package.json!',
            false,
            'yarn.lock',
            15_613,
            type: :error
          )])
      end

      it 'Errors when patch bump causes minor version changes' do
        allow(File).to receive(:readlines).with('yarn.lock')
          .and_return(
            load_fixture('dependencies/yarn.lock_patch_bumps_minor.lock')
            .split("\n")
          )

        allow(subject).to receive(:run_and_diff)
          .with(/yarn list/)
          .and_return(load_fixture('dependencies/yarn_patch_bumps_minor.diff'))
        allow(@git).to receive(:diff)
          .and_return(
            [
              load_diff(
                'spec/fixtures/dependencies/'\
                  'package.json_patch_bumps_minor.json',
                'dependencies/package.json_patch_bumps_minor'
              )
            ]
          )

        # act
        subject.perform

        expect(@dangerfile.violation_report[:warnings])
          .to eq([Violation.new(
            'Dangerous change! lodash was updated '\
            'from 4.17.15 to 4.18.1 without a corresponding '\
            'change to package.json!',
            false,
            'yarn.lock',
            # make sure we pick out the right lodash!  There's 3 of them.
            12_111,
            type: :error
          )])
      end

      it 'ignores all changes when top level major version changes' do
        allow(File).to receive(:readlines).with('yarn.lock')
          .and_return(
            load_fixture('dependencies/yarn.lock_second_level_effect.lock')
            .split("\n")
          )
        allow(File).to receive(:read).with('package.json')
          .and_return(
            load_fixture('dependencies/'\
              'package.json_second_level_effect.json')
          )

        allow(subject).to receive(:run_and_diff)
          .with(/yarn list/)
          .and_return(load_fixture(
                        'dependencies/yarn_list_second_level_effect.txt.diff'
                      ))
        allow(@git).to receive(:diff)
          .and_return(
            [
              load_diff(
                'package.json',
                'dependencies/package.json_second_level_effect'
              )
            ]
          )

        # act
        subject.perform

        expect(@dangerfile.violation_report[:warnings].map(&:message))
          .to eq([])
      end
    end
  end
end
